import * as pdfJSLib from 'pdfjs-dist'
import pdfJsWorker from 'pdfjs-dist/build/pdf.worker.entry'
import process from 'process'
import { getCurrentInstance, markRaw, reactive, ref, Ref, watchEffect } from 'vue'
import { fabric } from 'fabric'
import { IEvent } from 'fabric/fabric-impl'
import { DataRectangle, DrawRectangle, LabeledRect, SelectionIEvent } from '@/pdf-element-marker'
import FabricLabeledRect from '@/fabric/labeled.rect'
import rectangleDefaults from '@/fabric/rectangleDefaults'
import useVModelEmit from '@/composables/useVModelEmit'
import Color from 'color'

export default function useInitFabric (htmlCanvas: Ref<HTMLCanvasElement | undefined>) {
    const publicPath = process.env.BASE_URL ?? ''

    const instance = getCurrentInstance()
    const props = instance?.props
    const emit = instance?.emit
    const scale = window.devicePixelRatio

    const getPdfCanvas = async (file: string, pageNum: number, zoom: number) => {
        pdfJSLib.GlobalWorkerOptions.workerSrc = pdfJsWorker
        const loadingTask = pdfJSLib.getDocument(file)

        const pdf = await loadingTask.promise
        const page = await pdf.getPage(pageNum)
        const canvas = document.createElement('canvas')
        const context = canvas.getContext('2d') as CanvasRenderingContext2D
        const viewport = page.getViewport({ scale: zoom })
        canvas.height = viewport.height
        canvas.width = viewport.width
        const renderContext = {
            canvasContext: context,
            viewport: viewport,
        }

        viewport.width = canvas.width
        viewport.height = canvas.height

        if (emit) {
            emit('total-pages', pdf.numPages)
        }
        // For some reason, using await page.render(renderContext) causes a blank page so using promise instead
        return page.render(renderContext).promise.then(() => canvas)
    }

    let canvas = ref<fabric.Canvas | undefined>()
    let currentFile = ''
    let currentPage = 1
    let currentZoom = 1
    let currentWidth = 0
    let currentHeight = 0

    const setFile = async (canvas: fabric.Canvas, file: string, pageNum: number, zoom: number) => {
        let pdfCanvas
        if (currentFile !== file || currentPage !== pageNum) {
            const currentChildren = canvas.getObjects()
            canvas.remove(...currentChildren)
            currentFile = file
            pdfCanvas = await getPdfCanvas(file, pageNum, zoom * scale)
            currentWidth = pdfCanvas.width / zoom / scale
            currentHeight = pdfCanvas.height / zoom / scale
            canvas.setDimensions({ width: currentWidth, height: currentHeight })
            const pdfImage = new fabric.Image(pdfCanvas)
                .scaleToWidth(currentWidth)
                .set('selectable', false)
                .set('evented', false)

            canvas.add(pdfImage)
        }
    }

    const setZoom = (canvas: fabric.Canvas, zoom: number) => {
        if (currentZoom !== zoom) {
            const newZoom = zoom
            const zoomBy = newZoom / currentZoom
            const width = canvas.width ? canvas.width * zoomBy : 0
            const height = canvas.height ? canvas.height * zoomBy : 0
            currentWidth = width
            currentHeight = height
            canvas.setDimensions({ width, height })
            canvas.setZoom(newZoom)
            currentZoom = zoom
        }
    }

    const handleSelection = (event: SelectionIEvent | null) => {
        if (emit) {
            if (event?.selected.length) {
                emit('object-selected', event.selected[0])
            } else {
                emit('object-selected', null)
            }
        }
    }

    const vModelEmit = useVModelEmit(canvas)
    const initListeners = (canvas: fabric.Canvas) => {
        canvas.on('mouse:dblclick', (event: IEvent) => {
            if (emit && event?.target) {
                emit('object-dblclick', event?.target)
            }
        })
        canvas.on('selection:created', (event: IEvent) => {
            handleSelection(event as SelectionIEvent)
        })
        canvas.on('selection:updated', (event: IEvent) => {
            handleSelection(event as SelectionIEvent)
        })
        canvas.on('selection:cleared', () => {
            handleSelection(null)
        })
        canvas.on('object:modified', (event: IEvent) => {
            vModelEmit()
        })
    }

    const buildRestoreRect = (mv: DataRectangle[]) => {
        const existing = canvas.value?.getObjects('labeledRect') as LabeledRect[]
        if (!mv.length) {
            for (const labeledRect of existing) {
                canvas.value?.remove(labeledRect)
            }
            vModelEmit()
            return
        }

        for (const drawRectangle of mv) {
            if (!existing.find((rect) => {
                return rect.top === drawRectangle.top
                    && rect.left === drawRectangle.left
                    && rect.width === drawRectangle.width
                    && rect.height === drawRectangle.height
            })) {
                const newData = Object.assign(rectangleDefaults, drawRectangle)
                newData.fill = Color(newData.color).alpha(.7).rgb().toString()
                delete newData.isRotatable
                const newRect = new FabricLabeledRect(reactive(newData) as DrawRectangle,
                )
                newRect.setControlVisible('mtr', !!drawRectangle.isRotatable)
                canvas.value?.add(newRect)
                vModelEmit()
            }
        }

    }

    const initVModel = () => {
        watchEffect(() => {
            const mv = props?.modelValue as DataRectangle[]
            if (!mv || !canvas) {
                return
            }

            buildRestoreRect(mv)
        })
    }

    return async (file: string, pageNum: number, zoom: number) => {
        if (!htmlCanvas.value) {
            return
        }
        if (!canvas.value) {
            canvas.value = markRaw(new fabric.Canvas(htmlCanvas.value, {
                enableRetinaScaling: true,
                renderOnAddRemove: true,
                selection: false,
                uniformScaling: false,
            }))
            initListeners(canvas.value)
            initVModel()
        }
        await setFile(canvas.value, file, pageNum, zoom)
        setZoom(canvas.value, zoom)

        return canvas.value
    }
}
