import { getCurrentInstance, reactive, Ref, toRaw, watchEffect } from 'vue'
import { fabric } from 'fabric'
import { DrawRectangle, Rectangle } from '@/pdf-element-marker'
import { IEvent } from 'fabric/fabric-impl'
import LabeledRect from '@/fabric/labeled.rect'
import Color from 'color'
import rectangleDefaults from '@/fabric/rectangleDefaults'
import useVModelEmit from '@/composables/useVModelEmit'

export default function usePdfDrawRect (canvasRef: Ref<fabric.Canvas | undefined>) {

    const instance = getCurrentInstance()
    const props = instance?.props
    const emit = instance?.emit

    let rect: Rectangle | undefined
    let isDown = false
    let origX: number
    let origY: number
    let canvas: fabric.Canvas | undefined

    let isRotatable = false
    let color = Color('#FF0000')

    const vModelEmit = useVModelEmit(canvasRef)

    const mouseDown = (o: IEvent) => {
        if (!canvas) {
            return
        }
        canvas.setCursor('crosshair')
        isDown = true
        let pointer = canvas.getPointer(o.e)
        origX = pointer.x
        origY = pointer.y
        rect = new LabeledRect(reactive({
            ...rectangleDefaults,
            left: origX,
            top: origY,
            width: pointer.x - origX,
            height: pointer.y - origY,
            fill: color.alpha(.7).rgb().toString(),
        })) as Rectangle
        if (!isRotatable) {
            rect.setControlVisible('mtr', false)
        }

        canvas.add(rect)
    }

    const mouseMove = (o: IEvent) => {
        if (!isDown || !rect || !canvas) {
            return
        }
        const pointer = canvas.getPointer(o.e)

        if (origX > pointer.x) {
            rect.set({ left: Math.abs(pointer.x) })
        }
        if (origY > pointer.y) {
            rect.set({ top: Math.abs(pointer.y) })
        }

        rect.set({ width: Math.abs(origX - pointer.x) })
        rect.set({ height: Math.abs(origY - pointer.y) })

        canvas.requestRenderAll()
    }

    const mouseUp = () => {
        isDown = false
        if (!canvas || !emit || !rect) {
            return
        }
        if (rect.width < 10 || rect.height < 10) {
            canvas.remove(rect)
        } else {
            canvas.setActiveObject(rect)
        }
        canvas.setCursor('default')
        canvas.renderAll()

        emit('update:draw', null)
        // vModelEmit()
    }

    const stopDraw = () => {
        canvas?.off('mouse:down', mouseDown)
        canvas?.off('mouse:move', mouseMove)
        canvas?.off('mouse:up', mouseUp)
    }

    const startDraw = () => {
        canvas?.on('mouse:down', mouseDown)
        canvas?.on('mouse:move', mouseMove)
        canvas?.on('mouse:up', mouseUp)
    }

    const init = () => {
        if (!canvas) {
            canvas = toRaw(canvasRef.value) as fabric.Canvas
        }
    }

    watchEffect(() => {
        init()
        if (props?.draw === null) {
            stopDraw()
        }
        if (props?.draw !== null && props?.draw !== undefined && emit && canvasRef?.value) {
            const d = props.draw as DrawRectangle
            isRotatable = d.isRotatable || false
            if (d.color) {
                color = Color(d.color)
            }
            startDraw()
        }
    })
}
